# Copyright (C) 2010 by Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Configuration metadata for the various releases."""

from copy import copy

# Some URL parts
_LPL = "http://launchpadlibrarian.net"
_TBZR = "http://tortoisesvn.googlecode.com/svn/TortoiseOverlays/version-1.1.2/bin/TortoiseOverlays-1.1.2.19334-"


## Classes used to build a distribution ##

class Project(object):
    def __init__(self, lp_project, branch=None, revision=None):
        self.lp_project = lp_project
        self.branch = branch
        self.revision = revision
        self.update_url()
    
    def update_url(self):
        if self.branch:
            self.url = self.branch
        else:
            self.url = "lp:%s" % (self.lp_project,)
        if self.revision:
            self.url = "%s@%s" % (self.url, self.revision)
    
    def copy_write(self, **kargs):
        """Create a copy of this object, and change some of it attributes."""
        new = copy(self)
        new.write(**kargs)
        return new
    
    def write(self, **kargs):
        """Change some of this object's attributes."""
        for key, value in kargs.iteritems():
            setattr(self, key, value)
        self.update_url()

class Plugin(Project):
    def __init__(self, lp_project, description, branch=None, revision=None,
            core=False, display_in_installer=True):
        Project.__init__(self, lp_project, branch, revision)
        self.description = description
        self.core = core
        self.display_in_installer = display_in_installer
        self.update_id()
    
    def update_id(self):
        self.id = self.lp_project
        if self.id.startswith("bzr-"):
            self.id = self.id[4:]
    
    def write(self, **kargs):
        """Change some of this object's attributes."""
        super(Plugin, self).write(**kargs)
        self.update_url()

class Docset(object):
    def __init__(self, id, description, localization=False):
        self.id = id
        self.description = description
        self.localization = localization
        self.part_name = "bzr-chm-%s" % (id,)

class Download(object):
    def __init__(self, part_name, dest, url, flags):
        self.part_name = part_name
        self.dest = dest
        self.url = url
        self.flags = flags

class TortoiseOverlay(Download):
    def __init__(self, id):
        url = "%s%s.msi" % (_TBZR, id)
        Download.__init__(self, "tortoise-overlays-%s" % id,
            "tortoise-overlays", url, {'download-only': True})
        self.id = id

class Application(object):
    def __init__(self, id, name, description, params):
        self.id = id
        self.name = name
        self.description = description
        self.params = params

subvertpy = Project('subvertpy', branch='lp:~jelmer/subvertpy/trunk')
python_fastimport = Project('fastimport', branch='lp:python-fastimport')

launchpad = Plugin('launchpad', "Launchpad.net integration", core=True)
netrc_credential_store = Plugin('netrc_credential_store',
                                "netrc (credential store example)", core=True)
news_merge = Plugin('news_merge', "Smart merging of NEWS files", core=True)
bzrtools = Plugin('bzrtools', "BzrTools (miscellaneous additions)")
colo = Plugin('bzr-colo', "Colo (work with colocated branches)")
explorer = Plugin('bzr-explorer', "Bazaar Explorer", display_in_installer=False)
bzr_fastimport = Plugin('bzr-fastimport', "FastImport (import/export of history)")
loom = Plugin('bzr-loom', "Loom (manage a stack of patches over an upstream)")
pipeline = Plugin('bzr-pipeline',
                  "Pipeline (work on large changes as smaller pieces)")
rewrite = Plugin('bzr-rewrite', "Rewrite (change history by rebasing)")
svn = Plugin('bzr-svn', "Subversion 1.6 integration")
upload = Plugin('bzr-upload', "Upload working tree")
xmloutput = Plugin('bzr-xmloutput', "XmlOutput (interface for IDEs)")
qbzr = Plugin('qbzr', "QBzr (dialogs for GUI applications and IDEs)")


# The components in Bazaar 2.2
BAZAAR_2_2 = dict(
    name = 'bzr-2.2',

    # The projects to bundle
    projects = [
        Project('bzr', 'lp:bzr/2.2', revision='tag:bzr-2.2.3'),
        subvertpy.copy_write(revision='tag:subvertpy-0.7.5'),
        Project('tortoisebzr', revision='tag:release-0.5.8'),
        python_fastimport.copy_write(revision='tag:fastimport-0.9.0'),
        ],

    # The plugins to bundle
    plugins = [
        launchpad,
        netrc_credential_store,
        news_merge,
        bzrtools.copy_write(revision='tag:release-2.2.0'),
        colo.copy_write(revision='tag:0.2.0'),
        explorer.copy_write(revision='tag:release-1.1.2'),
        bzr_fastimport.copy_write(revision='tag:release-0.10.0'),
        loom.copy_write(revision='revno:127'), # Latest. No 2.2 release yet
        pipeline.copy_write(branch='lp:bzr-pipeline/stable',
                            revision='tag:release-1.1'),
        rewrite.copy_write(revision='tag:bzr-rewrite-0.6.1'),
        svn.copy_write(revision='tag:bzr-svn-1.0.4'),
        upload.copy_write(revision='tag:bzr-upload-1.0.0'),
        xmloutput.copy_write(revision='tag:0.8.6'),
        qbzr.copy_write(branch='lp:qbzr/0.19', revision='tag:release-0.19.3'),
        ],

    # These are the documentation sets shown in the installer
    docsets = [
        Docset('en', "Core Documentation"),
        Docset('developers', "Developer Documentation"),
        Docset('es', "Core Documentation - Spanish", localization=True),
        Docset('ja', "Core Documentation - Japanese", localization=True),
        Docset('ru', "Core Documentation - Russian", localization=True),
        ],

    # These are the applications shown in the installer
    applications = [
        Application("explorer", "Bazaar Explorer",
            "Bazaar Explorer GUI Application", "explorer"),
        ],

    # These are the additional dependencies we need to install.
    # (In the future, it would be nice to explicitly associate these with
    # the projects requiring them.)
    downloads = [
        Download('svn-lib', 'svn', _LPL + "/51246029/svn-win32-1.6.6.zip",
            {'strip-top-level-dir': True}),
        Download('svn-dev', 'svn', _LPL + "/51247521/svn-win32-1.6.6_dev.zip",
            {'strip-top-level-dir': True, 'ignore-existing': True}),
        Download('svn-db4', 'db4', _LPL + "/51247865/db-4.4.20-win32.zip",
            {'strip-top-level-dir': True}),
        Download('svn-libintl', 'libintl', _LPL + "/51249217/svn-win32-libintl.zip",
            {'strip-top-level-dir': True}),
        Download('zlib', 'zlib', _LPL + "/51249454/zlib123-dll.zip",
            {}),
        ],
    tortoise_overlays = [
        TortoiseOverlay('win32'),
        TortoiseOverlay('x64'),
        ],
    )


# The components in Bazaar 2.3
BAZAAR_2_3 = dict(
    name = 'bzr-2.3',

    # The projects to bundle
    projects = [
        Project('bzr', 'lp:bzr/2.3', revision='tag:bzr-2.3.3'),
        subvertpy.copy_write(revision='tag:subvertpy-0.7.5'),
        python_fastimport.copy_write(revision='tag:fastimport-0.9.0'),
        Project('tortoisebzr', revision='tag:release-0.6.3'),
        ],

    # The plugins to bundle
    plugins = [
        launchpad,
        netrc_credential_store,
        news_merge,
        bzrtools.copy_write(branch='lp:bzrtools/2.3',
                            revision='tag:release-2.3.1'),
        colo.copy_write(revision='tag:0.2.1'),
        explorer.copy_write(revision='tag:release-1.1.3'),
        bzr_fastimport.copy_write(revision='tag:release-0.10.0'),
        loom.copy_write(revision='revno:127'), # Latest. No 2.2 release yet
        pipeline.copy_write(branch='lp:bzr-pipeline/stable',
                            revision='tag:release-1.1'),
        rewrite.copy_write(revision='revno:227'),
        svn.copy_write(revision='revno:3515'),
        upload.copy_write(revision='tag:bzr-upload-1.0.0'),
        xmloutput.copy_write(revision='revno:149'),
        qbzr.copy_write(branch='lp:qbzr/0.20', revision='tag:release-0.20.1'),
        ],

    # These are the documentation sets shown in the installer
    docsets = BAZAAR_2_2['docsets'],

    # These are the applications shown in the installer
    applications = BAZAAR_2_2['applications'],

    # These are the additional dependencies we need to install.
    # (In the future, it would be nice to explicitly associate these with
    # the projects requiring them.)
    downloads = BAZAAR_2_2['downloads'],
    tortoise_overlays = BAZAAR_2_2['tortoise_overlays'],
    )

# The components in Bazaar 2.4
BAZAAR_2_4 = dict(
    name = 'bzr-2.4',

    # The projects to bundle
    projects = [
        Project('bzr', 'lp:bzr', revision='tag:bzr-2.4b2'),
        subvertpy.copy_write(revision='tag:subvertpy-0.8.0'),
        python_fastimport.copy_write(revision='tag:fastimport-0.9.0'),
        Project('tortoisebzr', revision='tag:release-0.6.3'),
        ],

    # The plugins to bundle
    plugins = [
        launchpad,
        netrc_credential_store,
        news_merge,
        bzrtools,
        colo,
        explorer.copy_write(revision='tag:release-1.1.3'),
        bzr_fastimport,
        loom,
        pipeline,
        rewrite,
        svn,
        upload,
        xmloutput.copy_write(revision='tag:0.8.7'),
        qbzr.copy_write(branch='lp:qbzr', revision='tag:release-0.21beta1'),
        ],

    # These are the documentation sets shown in the installer
    docsets = BAZAAR_2_2['docsets'],

    # These are the applications shown in the installer
    applications = BAZAAR_2_2['applications'],

    # These are the additional dependencies we need to install.
    # (In the future, it would be nice to explicitly associate these with
    # the projects requiring them.)
    downloads = BAZAAR_2_2['downloads'],
    tortoise_overlays = BAZAAR_2_2['tortoise_overlays'],
    )

# The components in Bazaar trunk
BAZAAR_TRUNK = dict(
    name = 'bzr-dev',

    # The projects to bundle
    projects = [
        Project('bzr'),
        subvertpy,
        Project('tortoisebzr'),
        ],

    # The plugins to bundle
    plugins = [
        launchpad,
        netrc_credential_store,
        news_merge,
        bzrtools,
        colo,
        explorer,
        bzr_fastimport,
        loom,
        pipeline,
        rewrite,
        svn,
        upload,
        xmloutput,
        qbzr,
        ],

    # These are the documentation sets shown in the installer
    docsets = BAZAAR_2_2['docsets'],

    # These are the applications shown in the installer
    applications = BAZAAR_2_2['applications'],

    # These are the additional dependencies we need to install.
    # (In the future, it would be nice to explicitly associate these with
    # the projects requiring them.)
    downloads = BAZAAR_2_2['downloads'],
    tortoise_overlays = BAZAAR_2_2['tortoise_overlays'],
    )

# The complete set of distributions. We don't go back before 2.2
# though because those installers are built with older scripts.
DISTRIBUTIONS = [
    BAZAAR_2_2,
    BAZAAR_2_3,
    BAZAAR_2_4,
    BAZAAR_TRUNK,
    ]

